@@ -1,5 +1,5 @@
 /* BEAST - Bedevilled Audio System
- * Copyright (C) 1998-2002 Tim Janik
+ * Copyright (C) 1998-2003 Tim Janik
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,14 +18,9 @@
 #ifndef __BST_ITEM_VIEW_H__
 #define __BST_ITEM_VIEW_H__
 
-#include	"bstdefs.h"
-#include	<gtk/gtk.h>
-
-
-#ifdef __cplusplus
-extern "C" {
-#endif /* __cplusplus */
+#include	"bstutils.h"
 
+G_BEGIN_DECLS
 
 /* --- Gtk+ type macros --- */
 #define	BST_TYPE_ITEM_VIEW	      (bst_item_view_get_type ())
@@ -35,6 +30,8 @@ extern "C" {
 #define	BST_IS_ITEM_VIEW_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), BST_TYPE_ITEM_VIEW))
 #define BST_ITEM_VIEW_GET_CLASS(obj)  (GTK_CHECK_GET_CLASS ((obj), BST_TYPE_ITEM_VIEW, BstItemViewClass))
 
+#define	BST_ITEM_VIEW_TREE_HEIGHT     (120)
+
 
 /* --- structures & typedefs --- */
 typedef	struct	_BstItemView		BstItemView;
@@ -42,68 +39,72 @@ typedef	struct	_BstItemViewClass	BstItemViewClass;
 typedef	struct	_BstItemViewOp		BstItemViewOp;
 struct _BstItemView
 {
-  GtkAlignment	 parent_object;
+  GtkAlignment	  parent_object;
 
-  GtkWidget	*paned;
+  GtkTreeView    *tree;
+  GxkListWrapper *wlist;
 
-  GType  	 item_type;
-  
-  BswProxy	 container;
-
-  gchar		*id_format;
-  guint		 item_list_pos;
-  
-  GtkWidget	*item_clist;
-  
-  GtkWidget	*param_view;
+  GtkPaned       *paned;
+  GtkWidget	 *pview;
+  GtkWidget	 *tools;
 
+  SfiProxy	 container;
+  SfiProxy	 auto_select;
   GtkWidget	**op_widgets;
 };
 struct _BstItemViewClass
 {
   GtkAlignmentClass parent_class;
 
+  const gchar	   *item_type;
   guint		    n_ops;
   BstItemViewOp	   *ops;
+  guint		    horizontal_ops : 1;
+  guint		    show_properties : 1;
 
-  guint             default_param_view_height;
-
-  void          (*operate)       (BstItemView	*item_view,
-				  BstOps	 op);
-  gboolean      (*can_operate)   (BstItemView	*item_view,
-				  BstOps         op);
+  void	      (*set_container)	(BstItemView	*self,
+				 SfiProxy	 new_container);
+  void	      (*create_tree)	(BstItemView	*self);
+  void	      (*listen_on)	(BstItemView	*self,
+				 SfiProxy	 item);
+  void	      (*unlisten_on)	(BstItemView	*self,
+				 SfiProxy	 item);
 };
 struct _BstItemViewOp
 {
-  gchar	      *op_name;
+  const gchar *op_name;
   guint	       op;
   const gchar *stock_icon;
+  const gchar *tooltip;
 };
 
 
 /* --- prototypes --- */
-GtkType		bst_item_view_get_type		(void);
-void		bst_item_view_rebuild		(BstItemView	*item_view);
-void		bst_item_view_update		(BstItemView	*item_view);
+GType		bst_item_view_get_type		(void);
 void		bst_item_view_select		(BstItemView	*item_view,
-						 BswProxy	 item);
-BswProxy	bst_item_view_get_current	(BstItemView	*item_view);
+						 SfiProxy	 item);
+SfiProxy	bst_item_view_get_current	(BstItemView	*item_view);
+SfiProxy	bst_item_view_get_proxy		(BstItemView	*item_view,
+						 gint            row);
+gint            bst_item_view_get_proxy_row     (BstItemView    *self,
+                                                 SfiProxy        item);
 void		bst_item_view_set_container	(BstItemView	*item_view,
-						 BswProxy	 new_container);
-void		bst_item_view_operate		(BstItemView	*item_view,
-						 BstOps		 op);
-gboolean	bst_item_view_can_operate	(BstItemView	*item_view,
-						 BstOps		 op);
-void		bst_item_view_set_id_format	(BstItemView	*item_view,
-						 const gchar	*id_format);
-void		bst_item_view_set_layout	(BstItemView	*item_view,
-						 gboolean	 horizontal,
-						 guint		 pos);
+						 SfiProxy	 new_container);
+void		bst_item_view_refresh   	(BstItemView    *self,
+						 SfiProxy        item);
+void		bst_item_view_name_edited	(BstItemView    *self,
+						 const gchar    *strpath,
+						 const gchar    *text);
+void		bst_item_view_blurb_edited	(BstItemView    *self,
+						 const gchar    *strpath,
+						 const gchar    *text);
+void		bst_item_view_enable_param_view	(BstItemView    *self,
+                                                 gboolean        enabled);
+GtkTreeModel* bst_item_view_adapt_list_wrapper	(BstItemView	*self,
+						 GxkListWrapper *lwrapper);
+void            bst_item_view_complete_tree     (BstItemView    *self);
 
 
-
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
+G_END_DECLS
 
 #endif /* __BST_ITEM_VIEW_H__ */
