@@ -20,9 +20,11 @@
 #include "bstapp.h"
 #include "bstsplash.h"
 #include "bstxkb.h"
+#include "bstactivatable.h"
 #include "bstgconfig.h"
 #include "bstusermessage.h"
 #include "bstcatalog.h"
+#include "bstparam.h"
 #include "bstpreferences.h"
 #include "PKG_config.h"
 #include "beast-gtk/images/beast-images.h"
@@ -40,11 +42,12 @@ static void			bst_print_blurb		(gboolean     print_help);
 
 /* --- variables --- */
 gboolean            bst_developer_hints = FALSE;
-gboolean            bst_developer_extensions = FALSE;
+gboolean            bst_debug_extensions = FALSE;
 gboolean            bst_main_loop_running = TRUE;
 static gboolean     registration_done = FALSE;
 static gboolean     arg_force_xkb = FALSE;
-static gboolean     register_plugins = TRUE;
+static gboolean     register_core_plugins = TRUE;
+static gboolean     register_ladspa_plugins = TRUE;
 static gboolean     register_scripts = TRUE;
 static const gchar *bst_rc_string =
 ( "style'BstTooltips-style'"
@@ -186,8 +189,8 @@ main (int   argc,
 		     "signal::registration", server_registration, splash,
 		     NULL);
 
-  /* register dynamic types and modules (plugins) */
-  if (register_plugins)
+  /* register core plugins */
+  if (register_core_plugins)
     {
       bst_splash_update_entity (splash, _("Plugins"));
 
@@ -195,7 +198,26 @@ main (int   argc,
        * so we wait until all are done
        */
       registration_done = FALSE;
-      bse_server_register_plugins (BSE_SERVER);
+      bse_server_register_core_plugins (BSE_SERVER);
+      while (!registration_done)
+	{
+	  GDK_THREADS_LEAVE ();
+	  g_main_iteration (TRUE);
+	  GDK_THREADS_ENTER ();
+	  sfi_glue_gc_run ();
+	}
+    }
+
+  /* register LADSPA plugins */
+  if (register_ladspa_plugins)
+    {
+      bst_splash_update_entity (splash, _("LADSPA Plugins"));
+
+      /* plugin registration, this is done asyncronously,
+       * so we wait until all are done
+       */
+      registration_done = FALSE;
+      bse_server_register_ladspa_plugins (BSE_SERVER);
       while (!registration_done)
 	{
 	  GDK_THREADS_LEAVE ();
@@ -261,7 +283,7 @@ main (int   argc,
 	  if (!app)
 	    {
 	      project = bse_server_use_new_project (BSE_SERVER, "Untitled.bse");
-	      wrepo = bse_project_ensure_wave_repo (project);
+	      wrepo = bse_project_get_wave_repo (project);
 	      error = bse_wave_repo_load_file (wrepo, argv[i]);
 	      if (!error)
 		{
@@ -275,7 +297,7 @@ main (int   argc,
 	    }
 	  else
 	    {
-	      SfiProxy wrepo = bse_project_ensure_wave_repo (app->project);
+	      SfiProxy wrepo = bse_project_get_wave_repo (app->project);
 	      
 	      gxk_status_printf (GXK_STATUS_WAIT, NULL, _("Loading \"%s\""), argv[i]);
 	      error = bse_wave_repo_load_file (wrepo, argv[i]);
@@ -305,7 +327,7 @@ main (int   argc,
     {
       SfiProxy project = bse_server_use_new_project (BSE_SERVER, "Untitled.bse");
       
-      bse_project_ensure_wave_repo (project);
+      bse_project_get_wave_repo (project);
       app = bst_app_new (project);
       bse_item_unuse (project);
       gxk_idle_show_widget (GTK_WIDGET (app));
@@ -317,7 +339,7 @@ main (int   argc,
    */
   if (!BST_RC_VERSION || strcmp (BST_RC_VERSION, BST_VERSION))
     {
-      bst_app_operate (app, BST_OP_HELP_RELEASE_NOTES);
+      bst_activatable_activate (BST_ACTIVATABLE (app), BST_ACTION_HELP_RELEASE_NOTES);
       bst_gconfig_set_rc_version (BST_VERSION);
     }
 
@@ -413,11 +435,14 @@ bst_early_parse_args (int    *argc_p,
 	  g_printerr ("BEAST(%s): pid = %u\n", BST_VERSION, getpid ());
 	  if (strchr (flags, 'n') != NULL)
 	    {
-	      register_plugins = FALSE;
+	      register_core_plugins = FALSE;
+	      register_ladspa_plugins = FALSE;
 	      register_scripts = FALSE;
 	    }
 	  if (strchr (flags, 'p'))
-	    register_plugins = TRUE;
+	    register_core_plugins = TRUE;
+	  if (strchr (flags, 'l'))
+	    register_ladspa_plugins = TRUE;
 	  if (strchr (flags, 's'))
 	    register_scripts = TRUE;
 	  if (strchr (flags, 'f'))
@@ -428,10 +453,10 @@ bst_early_parse_args (int    *argc_p,
 	    }
 	  if (strchr (flags, 'd') != NULL)
 	    {
-	      g_message ("enabling possibly harmfull developer functionality due to '-:d'");
-	      bst_developer_extensions = strchr (flags, 'd') != NULL;
+	      g_message ("enabling possibly harmfull debugging functionality due to '-:d'");
+	      bst_debug_extensions = strchr (flags, 'd') != NULL;
 	    }
-	  sfi_rec_set_bool (bseconfig, "developer-extensions", bst_developer_extensions);
+	  sfi_rec_set_bool (bseconfig, "debug-extensions", bst_debug_extensions);
 	  argv[i] = NULL;
 	}
       else if (strcmp ("--hints", argv[i]) == 0)
@@ -553,7 +578,7 @@ bst_print_blurb (gboolean print_help)
       g_print ("Image Path:  %s\n", BST_PATH_IMAGES);
       g_print ("Plugin Path: %s\n", BSE_PATH_PLUGINS);
       g_print ("Script Path: %s\n", BSE_PATH_SCRIPTS);
-      g_print ("Sample Path: %s\n", BST_PATH_DATA_SAMPLES);
+      g_print ("Sample Path: %s:$BEAST_SAMPLE_PATH\n", BST_PATH_DATA_SAMPLES);
       g_print ("LADSPA Path: %s:$LADSPA_PATH\n", BSE_PATH_LADSPA);
       g_print ("\n");
       g_print ("BEAST comes with ABSOLUTELY NO WARRANTY.\n");
@@ -579,9 +604,10 @@ bst_print_blurb (gboolean print_help)
       g_print ("  -:[flags]               [flags] can be any of:\n");
       g_print ("                          f - fatal warnings\n");
       g_print ("                          n - disable script and plugin registration\n");
-      g_print ("                          p - reenable plugin registration\n");
+      g_print ("                          p - reenable core plugin registration\n");
+      g_print ("                          l - reenable LADSPA plugin registration\n");
       g_print ("                          s - reenable script registration\n");
-      g_print ("                          d - development extensions (harmfull)\n");
+      g_print ("                          d - debugging extensions (harmfull)\n");
       if (!BST_VERSION_STABLE)
         {
 	  g_print ("Gtk+ Options:\n");
