@@ -31,14 +31,19 @@ G_BEGIN_DECLS
 
 
 /* --- Gtk+ workarounds and amendments --- */
-gboolean    gxk_cell_editable_focus_out_handler (GtkCellEditable *ecell);
+void        gxk_cell_editable_is_focus_handler  (GtkCellEditable *ecell);
 gboolean    gxk_cell_editable_canceled		(GtkCellEditable *ecell);
 GtkWidget*  gxk_item_factory_get_item		(GtkItemFactory	 *ifactory,
 						 const gchar	 *path);
 GtkWidget*  gxk_item_factory_get_widget		(GtkItemFactory	 *ifactory,
 						 const gchar	 *path);
 void	    gxk_widget_proxy_requisition	(GtkWidget	 *widget);
-void	    gxk_file_selection_heal		(GtkFileSelection *fs);
+
+
+/* --- GtkFileSelection workarounds --- */
+GtkWidget*	gxk_file_selection_heal		(GtkFileSelection	 *fs);
+GtkWidget*	gxk_file_selection_split	(GtkFileSelection	 *fs,
+						 GtkWidget		**bbox);
 
 
 /* --- GObject convenience --- */
@@ -64,6 +69,16 @@ glong	g_object_get_long		(gpointer		 object,
 #define	g_object_get_int		g_object_get_long	// FIXME
 
 
+/* --- UTF8 helpers --- */
+gchar*	gxk_convert_latin1_to_utf8	(const gchar	*string);
+gchar*	gxk_filename_to_utf8		(const gchar	*string);
+
+
+/* --- Gtk+ Utilities --- */
+void	 gxk_widget_viewable_changed	(GtkWidget		*widget);
+gboolean gxk_widget_viewable		(GtkWidget		*widget);
+
+
 /* --- Gdk convenience --- */
 #define	GXK_DEFAULT_CURSOR	GDK_LAST_CURSOR	/* revert to default (inherited) cursor */
 void	gxk_window_set_cursor_type	(GdkWindow		*window,
@@ -80,6 +95,11 @@ void	gxk_widget_make_insensitive	(GtkWidget	*widget);
 void	gxk_widget_make_sensitive	(GtkWidget	*widget);
 void	gxk_widget_showraise		(GtkWidget	*widget);
 void	gxk_idle_show_widget		(GtkWidget	*widget);
+void	gxk_idle_unrealize_widget	(GtkWidget	*widget);
+void    gxk_notebook_add_page		(GtkNotebook	*notebook,
+					 GtkWidget	*child,
+					 const gchar	*tab_text,
+					 gboolean        expand_fill);
 void    gxk_notebook_set_current_page_widget (GtkNotebook *notebook,
 					      GtkWidget   *page);
 GtkWidget* gtk_notebook_current_widget       (GtkNotebook *notebook);
@@ -114,31 +134,38 @@ void	 gxk_tree_view_append_text_columns	(GtkTreeView		*tree_view,
 						 ...);
 void	 gxk_tree_view_add_text_column		(GtkTreeView  *tree_view,
 						 guint	       model_column,
+						 const gchar  *column_flags,
 						 gdouble       xalign,
 						 const gchar  *title,
 						 const gchar  *tooltip,
 						 gpointer      edited_callback,
 						 gpointer      data,
 						 GConnectFlags cflags);
+void	 gxk_tree_view_add_popup_column		(GtkTreeView  *tree_view,
+						 guint	       model_column,
+						 const gchar  *column_flags,
+						 gdouble       xalign,
+						 const gchar  *title,
+						 const gchar  *tooltip,
+						 gpointer      edited_callback,
+						 gpointer      popup_callback,
+						 gpointer      data,
+						 GConnectFlags cflags);
 void	gxk_tree_view_add_toggle_column		(GtkTreeView  *tree_view,
-						 guint	      model_column,
+						 guint	       model_column,
+						 const gchar  *column_flags,
 						 gdouble       xalign,
 						 const gchar  *title,
 						 const gchar  *tooltip,
 						 gpointer      toggled_callback,
 						 gpointer      data,
 						 GConnectFlags cflags);
-void	 gxk_tree_view_add_popup_column		(GtkTreeView  *tree_view,
-						 guint	       model_column,
-						 gdouble       xalign,
-						 const gchar  *title,
-						 const gchar  *tooltip,
-						 gpointer      edited_callback,
-						 gpointer      data,
-						 GConnectFlags cflags);
 void	gxk_tree_view_column_set_tip_title	(GtkTreeViewColumn   *tree_column,
 						 const gchar         *title,
 						 const gchar	     *tooltip);
+void	gxk_tree_view_set_editable      	(GtkTreeView         *tview,
+                                                 gboolean             maybe_editable);
+
 
 /* tree selection convenience */
 void   gxk_tree_selection_select_spath   (GtkTreeSelection      *selection,
