@@ -1,5 +1,5 @@
 /* BSE - Bedevilled Sound Engine
- * Copyright (C) 1998, 1999 Olaf Hoehmann and Tim Janik
+ * Copyright (C) 1998-2002 Tim Janik
  *
  * This library is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,20 +31,25 @@ extern "C" {
 
 /* --- plugin export macros --- */
 /* (implementations reside in bseplugin.h) */
-/* start export section,
- * provide a unique plugin here
+/* start export section, provide a unique plugin name here
  */
 #define BSE_EXPORTS_BEGIN(UniqueName)   BSE_EXPORT_IMPL_B (UniqueName)
-/* list procedure types as BseExportProcedure array
+/* list procedure types in BseExportProcedure array
  */
 #define BSE_EXPORT_PROCEDURES           BSE_EXPORT_IMPL_A (Procedure)
-/* list object types as BseExportObject array
+/* list object types in BseExportObject array
  */
 #define BSE_EXPORT_OBJECTS              BSE_EXPORT_IMPL_A (Object)
-/* list enum types as BseExportEnum array
+/* qualify exported procedure types as file handlers
+ */
+#define BSE_EXPORT_FILE_HANDLERS        BSE_EXPORT_IMPL_A (FileHandler)
+/* list enum types as BseExportEnum array (mere internal use)
  */
 #define BSE_EXPORT_STATIC_ENUMS		static const BseExportEnum \
                                         BSE_EXPORT_IMPL_S (MkEnums_built) []
+/* directive, used to trigger automated enum generation from
+ * plugin's .h file. also auto-exports them to BSE
+ */
 #define BSE_EXPORT_AND_GENERATE_ENUMS() BSE_EXPORT_IMPL_P (Enum) = \
                                         BSE_EXPORT_IMPL_S (MkEnums_built)
 /* end export section
@@ -55,14 +60,14 @@ extern "C" {
 /* --- typedefs --- */
 typedef const gchar*                        BseExportBegin;
 typedef union  _BseExportSpec               BseExportSpec;
-typedef struct _BseExportAny                BseExportAny;
 typedef struct _BseExportObject             BseExportObject;
 typedef struct _BseExportEnum               BseExportEnum;
+typedef struct _BseExportFileHandler   	    BseExportFileHandler;
 typedef struct _BseExportProcedure     	    BseExportProcedure;
 typedef guint                               BseExportEnd;
 typedef void         (*BseProcedureInit)   (BseProcedureClass *proc,
-					    BseParamSpec     **ipspecs,
-					    BseParamSpec     **opspecs);
+					    GParamSpec	     **in_pspecs,
+					    GParamSpec	     **out_pspecs);
 typedef void         (*BseProcedureUnload) (BseProcedureClass *procedure);
 
 
@@ -71,69 +76,82 @@ typedef enum			/*< skip >*/
 {
   BSE_EXPORT_TYPE_PROCS		= 1,
   BSE_EXPORT_TYPE_OBJECTS	= 2,
-  BSE_EXPORT_TYPE_ENUMS		= 3
+  BSE_EXPORT_TYPE_ENUMS		= 3,
+  BSE_EXPORT_TYPE_FILE_HANDLERS	= 4
 } BseExportType;
 
 
-/* --- export declarations --- */
-struct _BseExportAny
+/* --- File Handler Types --- */
+typedef enum
 {
-  BseType            *type_p;
-  const gchar  	     *name;
-};
+  BSE_FILE_CUSTOM_LOADER	= -100,
+  BSE_FILE_STANDARD_LOADER	= 0,
+  BSE_FILE_FALLBACK_LOADER	= 100,
+} BseFileHandlerType;
+
+
+/* --- export declarations --- */
 struct _BseExportProcedure
 {
-  BseType            *type_p;	   /* obligatory */
+  GType              *type_p;	   /* obligatory */
   const gchar  	     *name;	   /* obligatory */
   const gchar  	     *blurb;	   /* optional */
-  const guint  	      private_id;  /* optional */
+  guint  	      private_id;  /* optional */
 
   BseProcedureInit    init;	   /* obligatory */
   BseProcedureExec    exec;	   /* obligatory */
   BseProcedureUnload  unload;	   /* optional */
   
   const gchar  	     *category;	   /* recommended */
-  const BsePixdata    pixdata;     /* optional */
+  BsePixdata	      pixdata;     /* optional */
 };
 struct _BseExportObject
 {
-  BseType            *type_p;	   /* obligatory */
+  GType              *type_p;	   /* obligatory */
   const gchar  	     *name;	   /* obligatory */
   const gchar  	     *parent_type; /* obligatory */
   const gchar  	     *blurb;	   /* optional */
 
-  const BseTypeInfo  *object_info; /* obligatory */
+  const GTypeInfo  *object_info; /* obligatory */
 
   const gchar  	     *category;	   /* recommended */
-  const BsePixdata    pixdata;     /* optional */
+  BsePixdata	      pixdata;     /* optional */
 };
 struct _BseExportEnum
 {
-  BseType            *type_p;	   /* obligatory */
+  GType              *type_p;	   /* obligatory */
   const gchar  	     *name;	   /* obligatory */
-  BseType             parent_type; /* obligatory */
+  GType               parent_type; /* obligatory */
   gpointer            values;      /* obligatory */
 };
+struct _BseExportFileHandler
+{
+  GType              *type_p;      /* obligatory, referring to procedure type */
+  BseFileHandlerType  fh_type;
+  const gchar        *prefix;	   /* optional */
+  const gchar	     *extension;   /* optional */
+  const gchar        *magic;	   /* optional, prerequisite if given */
+};
 
 
 /* --- export union --- */
 union _BseExportSpec
 {
-  BseType		 *type_p; /* common to all members */
-  BseExportAny		  any;
+  GType  		 *type_p; /* common to all members */
   BseExportProcedure	  s_proc;
   BseExportObject	  s_object;
   BseExportEnum		  s_enum;
+  BseExportFileHandler	  s_file_handler;
 };
 
 
 /* --- internal prototypes --- */
 void	bse_procedure_complete_info	(const BseExportSpec *spec,
-					 BseTypeInfo         *info);
+					 GTypeInfo         *info);
 void	bse_object_complete_info	(const BseExportSpec *spec,
-					 BseTypeInfo         *info);
+					 GTypeInfo         *info);
 void	bse_enum_complete_info		(const BseExportSpec *spec,
-					 BseTypeInfo         *info);
+					 GTypeInfo         *info);
 
 
 
