@@ -4,14 +4,10 @@
 
 GType BSE_TYPE_ID (BseFileHandlerType) = 0;
 
-GType BSE_TYPE_ID (BseBinDataType) = 0;
-
 GType BSE_TYPE_ID (BseBiquadFilterType) = 0;
 
 GType BSE_TYPE_ID (BseBiquadFilterNorm) = 0;
 
-GType BSE_TYPE_ID (BseEndianType) = 0;
-
 GType BSE_TYPE_ID (BseRegistrationType) = 0;
 
 GType BSE_TYPE_ID (BseInterpolType) = 0;
@@ -47,10 +43,6 @@ GType BSE_TYPE_ID (BseStandardOscWaveType) = 0;
 extern BSE_BUILTIN_PROTO (BseAmplifier);
 GType bse_type_builtin_id_BseAmplifier = 0;
 
-/* --- bsebindata.c --- */
-extern BSE_BUILTIN_PROTO (BseBinData);
-GType bse_type_builtin_id_BseBinData = 0;
-
 /* --- bsebiquadfilter.c --- */
 extern BSE_BUILTIN_PROTO (BseBiquadFilter);
 GType bse_type_builtin_id_BseBiquadFilter = 0;
@@ -215,6 +207,10 @@ GType bse_type_builtin_id_BseWave = 0;
 extern BSE_BUILTIN_PROTO (BseWaveOsc);
 GType bse_type_builtin_id_BseWaveOsc = 0;
 
+/* --- bsecsynth.c --- */
+extern BSE_BUILTIN_PROTO (BseCSynth);
+GType bse_type_builtin_id_BseCSynth = 0;
+
 /* --- bsewaverepo.c --- */
 extern BSE_BUILTIN_PROTO (BseWaveRepo);
 GType bse_type_builtin_id_BseWaveRepo = 0;
@@ -223,6 +219,10 @@ GType bse_type_builtin_id_BseWaveRepo = 0;
 extern BSE_BUILTIN_PROTO (BseLadspaModule);
 GType bse_type_builtin_id_BseLadspaModule = 0;
 
+/* --- bsepcmwriter.c --- */
+extern BSE_BUILTIN_PROTO (BsePcmWriter);
+GType bse_type_builtin_id_BsePcmWriter = 0;
+
 /**
  ** Generated data ends here
  **/
@@ -230,6 +230,7 @@ GType bse_type_builtin_id_BseLadspaModule = 0;
 /*-------- begin ../sfi/sfidl generated code --------*/
 
 
+#include <string.h>
 BseIntSeq*
 bse_int_seq_new (void)
 {
@@ -324,6 +325,7 @@ bse_int_seq_free (BseIntSeq* seq)
 {
   g_return_if_fail (seq != NULL);
   
+  g_free (seq->ints);
   g_free (seq);
 }
 
@@ -422,6 +424,7 @@ bse_note_seq_free (BseNoteSeq* seq)
 {
   g_return_if_fail (seq != NULL);
   
+  g_free (seq->notes);
   g_free (seq);
 }
 
@@ -530,6 +533,7 @@ bse_string_seq_free (BseStringSeq* seq)
   
   for (i = 0; i < seq->n_strings; i++)
     g_free (seq->strings[i]);
+  g_free (seq->strings);
   g_free (seq);
 }
 
@@ -628,6 +632,7 @@ bse_proxy_seq_free (BseProxySeq* seq)
 {
   g_return_if_fail (seq != NULL);
   
+  g_free (seq->proxies);
   g_free (seq);
 }
 
@@ -740,6 +745,7 @@ bse_part_note_seq_free (BsePartNoteSeq* seq)
   
   for (i = 0; i < seq->n_pnotes; i++)
     bse_part_note_free (seq->pnotes[i]);
+  g_free (seq->pnotes);
   g_free (seq);
 }
 
@@ -852,6 +858,7 @@ bse_category_seq_free (BseCategorySeq* seq)
   
   for (i = 0; i < seq->n_cats; i++)
     bse_category_free (seq->cats[i]);
+  g_free (seq->cats);
   g_free (seq);
 }
 
@@ -964,6 +971,7 @@ bse_dot_seq_free (BseDotSeq* seq)
   
   for (i = 0; i < seq->n_dots; i++)
     bse_dot_free (seq->dots[i]);
+  g_free (seq->dots);
   g_free (seq);
 }
 
@@ -1076,6 +1084,7 @@ bse_track_part_seq_free (BseTrackPartSeq* seq)
   
   for (i = 0; i < seq->n_tparts; i++)
     bse_track_part_free (seq->tparts[i]);
+  g_free (seq->tparts);
   g_free (seq);
 }
 
@@ -1090,10 +1099,10 @@ bse_part_note_new (void)
 BsePartNote*
 bse_part_note_copy_shallow (BsePartNote* rec)
 {
-  BsePartNote* rec_copy = NULL;
+  BsePartNote* rec_copy;
 
   g_return_val_if_fail (rec != NULL, NULL);
-  rec_copy = bse_part_note_new ();
+  rec_copy = g_new0 (BsePartNote, 1);
   rec_copy->id = rec->id;
   rec_copy->tick = rec->tick;
   rec_copy->duration = rec->duration;
@@ -1198,10 +1207,10 @@ bse_note_description_new (void)
 BseNoteDescription*
 bse_note_description_copy_shallow (BseNoteDescription* rec)
 {
-  BseNoteDescription* rec_copy = NULL;
+  BseNoteDescription* rec_copy;
 
   g_return_val_if_fail (rec != NULL, NULL);
-  rec_copy = bse_note_description_new ();
+  rec_copy = g_new0 (BseNoteDescription, 1);
   rec_copy->note = rec->note;
   rec_copy->octave = rec->octave;
   rec_copy->freq = rec->freq;
@@ -1329,10 +1338,10 @@ bse_note_sequence_new (void)
 BseNoteSequence*
 bse_note_sequence_copy_shallow (BseNoteSequence* rec)
 {
-  BseNoteSequence* rec_copy = NULL;
+  BseNoteSequence* rec_copy;
 
   g_return_val_if_fail (rec != NULL, NULL);
-  rec_copy = bse_note_sequence_new ();
+  rec_copy = g_new0 (BseNoteSequence, 1);
   rec_copy->offset = rec->offset;
   rec_copy->notes = bse_note_seq_copy_shallow (rec->notes);
   return rec_copy;
@@ -1406,10 +1415,10 @@ bse_icon_new (void)
 BseIcon*
 bse_icon_copy_shallow (BseIcon* rec)
 {
-  BseIcon* rec_copy = NULL;
+  BseIcon* rec_copy;
 
   g_return_val_if_fail (rec != NULL, NULL);
-  rec_copy = bse_icon_new ();
+  rec_copy = g_new0 (BseIcon, 1);
   rec_copy->bytes_per_pixel = rec->bytes_per_pixel;
   rec_copy->width = rec->width;
   rec_copy->height = rec->height;
@@ -1497,10 +1506,10 @@ bse_category_new (void)
 BseCategory*
 bse_category_copy_shallow (BseCategory* rec)
 {
-  BseCategory* rec_copy = NULL;
+  BseCategory* rec_copy;
 
   g_return_val_if_fail (rec != NULL, NULL);
-  rec_copy = bse_category_new ();
+  rec_copy = g_new0 (BseCategory, 1);
   rec_copy->category_id = rec->category_id;
   rec_copy->category = g_strdup (rec->category);
   rec_copy->mindex = rec->mindex;
@@ -1603,10 +1612,10 @@ bse_dot_new (void)
 BseDot*
 bse_dot_copy_shallow (BseDot* rec)
 {
-  BseDot* rec_copy = NULL;
+  BseDot* rec_copy;
 
   g_return_val_if_fail (rec != NULL, NULL);
-  rec_copy = bse_dot_new ();
+  rec_copy = g_new0 (BseDot, 1);
   rec_copy->x = rec->x;
   rec_copy->y = rec->y;
   return rec_copy;
@@ -1676,10 +1685,10 @@ bse_track_part_new (void)
 BseTrackPart*
 bse_track_part_copy_shallow (BseTrackPart* rec)
 {
-  BseTrackPart* rec_copy = NULL;
+  BseTrackPart* rec_copy;
 
   g_return_val_if_fail (rec != NULL, NULL);
-  rec_copy = bse_track_part_new ();
+  rec_copy = g_new0 (BseTrackPart, 1);
   rec_copy->tick = rec->tick;
   rec_copy->part = rec->part;
   rec_copy->duration = rec->duration;
@@ -1756,10 +1765,10 @@ bse_song_timing_new (void)
 BseSongTiming*
 bse_song_timing_copy_shallow (BseSongTiming* rec)
 {
-  BseSongTiming* rec_copy = NULL;
+  BseSongTiming* rec_copy;
 
   g_return_val_if_fail (rec != NULL, NULL);
-  rec_copy = bse_song_timing_new ();
+  rec_copy = g_new0 (BseSongTiming, 1);
   rec_copy->tick = rec->tick;
   rec_copy->bpm = rec->bpm;
   rec_copy->nominator = rec->nominator;
@@ -1847,6 +1856,113 @@ bse_song_timing_free (BseSongTiming* rec)
 }
 
 
+BseSampleFileInfo*
+bse_sample_file_info_new (void)
+{
+  BseSampleFileInfo* rec = g_new0 (BseSampleFileInfo, 1);
+  rec->waves = bse_string_seq_new ();
+  return rec;
+}
+
+BseSampleFileInfo*
+bse_sample_file_info_copy_shallow (BseSampleFileInfo* rec)
+{
+  BseSampleFileInfo* rec_copy;
+
+  g_return_val_if_fail (rec != NULL, NULL);
+  rec_copy = g_new0 (BseSampleFileInfo, 1);
+  rec_copy->file = g_strdup (rec->file);
+  rec_copy->size = rec->size;
+  rec_copy->mtime = rec->mtime;
+  rec_copy->loader = g_strdup (rec->loader);
+  rec_copy->waves = bse_string_seq_copy_shallow (rec->waves);
+  rec_copy->error = rec->error;
+  return rec_copy;
+}
+
+static BseSampleFileInfo*
+bse_sample_file_info_copy_shallow_internal (BseSampleFileInfo* rec)
+{
+  BseSampleFileInfo* rec_copy = NULL;
+  if (rec)
+    rec_copy = bse_sample_file_info_copy_shallow (rec);
+  return rec_copy;
+}
+
+BseSampleFileInfo*
+bse_sample_file_info_from_rec (SfiRec *sfi_rec)
+{
+  GValue *element;
+  BseSampleFileInfo* rec;
+
+  g_return_val_if_fail (sfi_rec != NULL, NULL);
+
+  rec = g_new0 (BseSampleFileInfo, 1);
+  element = sfi_rec_get (sfi_rec, "file");
+  if (element)
+    rec->file = g_strdup (sfi_value_get_string (element));
+  element = sfi_rec_get (sfi_rec, "size");
+  if (element)
+    rec->size = sfi_value_get_int (element);
+  element = sfi_rec_get (sfi_rec, "mtime");
+  if (element)
+    rec->mtime = sfi_value_get_num (element);
+  element = sfi_rec_get (sfi_rec, "loader");
+  if (element)
+    rec->loader = g_strdup (sfi_value_get_string (element));
+  element = sfi_rec_get (sfi_rec, "waves");
+  if (element)
+    rec->waves = bse_string_seq_from_seq (sfi_value_get_seq (element));
+  else
+    rec->waves = bse_string_seq_new ();
+  element = sfi_rec_get (sfi_rec, "error");
+  if (element)
+    rec->error = sfi_value_get_int (element);
+  return rec;
+}
+
+SfiRec *
+bse_sample_file_info_to_rec (BseSampleFileInfo* rec)
+{
+  SfiRec *sfi_rec;
+  GValue *element;
+
+  g_return_val_if_fail (rec != NULL, NULL);
+
+  sfi_rec = sfi_rec_new ();
+  element = sfi_value_string (rec->file);
+  sfi_rec_set (sfi_rec, "file", element);
+  sfi_value_free (element);
+  element = sfi_value_int (rec->size);
+  sfi_rec_set (sfi_rec, "size", element);
+  sfi_value_free (element);
+  element = sfi_value_num (rec->mtime);
+  sfi_rec_set (sfi_rec, "mtime", element);
+  sfi_value_free (element);
+  element = sfi_value_string (rec->loader);
+  sfi_rec_set (sfi_rec, "loader", element);
+  sfi_value_free (element);
+  element = sfi_value_seq_take_ref (bse_string_seq_to_seq (rec->waves));
+  sfi_rec_set (sfi_rec, "waves", element);
+  sfi_value_free (element);
+  element = sfi_value_int (rec->error);
+  sfi_rec_set (sfi_rec, "error", element);
+  sfi_value_free (element);
+  return sfi_rec;
+}
+
+void
+bse_sample_file_info_free (BseSampleFileInfo* rec)
+{
+  g_return_if_fail (rec != NULL);
+  
+  g_free (rec->file);
+  g_free (rec->loader);
+  bse_string_seq_free (rec->waves);
+  g_free (rec);
+}
+
+
 BseGConfig*
 bse_gconfig_new (void)
 {
@@ -1857,10 +1973,10 @@ bse_gconfig_new (void)
 BseGConfig*
 bse_gconfig_copy_shallow (BseGConfig* rec)
 {
-  BseGConfig* rec_copy = NULL;
+  BseGConfig* rec_copy;
 
   g_return_val_if_fail (rec != NULL, NULL);
-  rec_copy = bse_gconfig_new ();
+  rec_copy = g_new0 (BseGConfig, 1);
   rec_copy->synth_block_size = rec->synth_block_size;
   rec_copy->synth_mixing_freq = rec->synth_mixing_freq;
   rec_copy->invert_sustain = rec->invert_sustain;
@@ -2160,6 +2276,33 @@ static SfiBoxedRecordInfo bse_song_timing_boxed_info = {
 };
 GType BSE_TYPE_SONG_TIMING = 0;
 
+static GParamSpec *bse_sample_file_info_field[6];
+SfiRecFields bse_sample_file_info_fields = { 6, bse_sample_file_info_field };
+static void
+bse_sample_file_info_boxed2rec (const GValue *src_value, GValue *dest_value)
+{
+  gpointer boxed = g_value_get_boxed (src_value);
+  sfi_value_take_rec (dest_value, boxed ? bse_sample_file_info_to_rec (boxed) : NULL);
+}
+static void
+bse_sample_file_info_rec2boxed (const GValue *src_value, GValue *dest_value)
+{
+  SfiRec *rec = sfi_value_get_rec (src_value);
+  g_value_set_boxed_take_ownership (dest_value,
+    rec ? bse_sample_file_info_from_rec (rec) : NULL);
+}
+static const gchar *bse_sample_file_info_info_strings[] = {
+  NULL,
+};
+static SfiBoxedRecordInfo bse_sample_file_info_boxed_info = {
+  "BseSampleFileInfo",
+  { 6, bse_sample_file_info_field },
+  bse_sample_file_info_boxed2rec,
+  bse_sample_file_info_rec2boxed,
+  bse_sample_file_info_info_strings
+};
+GType BSE_TYPE_SAMPLE_FILE_INFO = 0;
+
 static GParamSpec *bse_gconfig_field[5];
 SfiRecFields bse_gconfig_fields = { 5, bse_gconfig_field };
 static void
@@ -2465,6 +2608,13 @@ _sfidl_types_init (void)
   bse_song_timing_field[4] = sfidl_pspec_Int (NULL,"tpqn","Ticks per quarter note" , NULL , 384 , 1 , G_MAXINT , 12 , SFI_PARAM_DEFAULT);
   bse_song_timing_field[5] = sfidl_pspec_Int (NULL,"tpt","Ticks per tact" , NULL , 384 , 1 , G_MAXINT , 12 , SFI_PARAM_DEFAULT);
 
+  bse_sample_file_info_field[0] = sfidl_pspec_String (NULL,"file","Filename" , NULL , NULL , SFI_PARAM_DEFAULT);
+  bse_sample_file_info_field[1] = sfidl_pspec_Int (NULL,"size","File Size" , NULL , 0 , 0 , G_MAXINT , 0 , SFI_PARAM_DEFAULT);
+  bse_sample_file_info_field[2] = sfidl_pspec_Num (NULL,"mtime","Modification Time" , NULL , 0 , 0 , SFI_MAXNUM , 0 , SFI_PARAM_DEFAULT);
+  bse_sample_file_info_field[3] = sfidl_pspec_String (NULL,"loader","Loader" , NULL , NULL , SFI_PARAM_DEFAULT);
+  bse_sample_file_info_field[4] = sfidl_pspec_BoxedSeq (NULL,"waves",NULL , NULL , SFI_PARAM_DEFAULT,bse_string_seq_content);
+  bse_sample_file_info_field[5] = sfidl_pspec_Int (NULL,"error","BseErrorType" , NULL , 0 , 0 , G_MAXINT , 0 , SFI_PARAM_DEFAULT);
+
   bse_gconfig_field[0] = sfidl_pspec_Int ("Synthesis Settings","synth_block_size","Synth Block Size" , "Synthesis buffer size, smaller values may reduce latency but " "increase CPU overhead" , 256 , 4 , 4096 , 4 , SFI_PARAM_DEFAULT "" SFI_PARAM_HINT_SCALE);
   bse_gconfig_field[1] = sfidl_pspec_Int ("Synthesis Settings","synth_mixing_freq","Synth Mixing Frequency" , "Synthesis mixing frequency, common values are: 22050, 44100, 48000" , 44100 , 8000 , 192000 , 0 , SFI_PARAM_DEFAULT);
   bse_gconfig_field[2] = sfidl_pspec_Bool ("MIDI","invert_sustain","Invert Sustain Pedal" , "Invert the state of sustain (damper) pedal so on/off meanings are reversed" , FALSE , SFI_PARAM_DEFAULT);
@@ -2494,6 +2644,9 @@ _sfidl_types_init (void)
   BSE_TYPE_SONG_TIMING = sfi_boxed_make_record (&bse_song_timing_boxed_info,
     (GBoxedCopyFunc) bse_song_timing_copy_shallow_internal,
     (GBoxedFreeFunc) bse_song_timing_free);
+  BSE_TYPE_SAMPLE_FILE_INFO = sfi_boxed_make_record (&bse_sample_file_info_boxed_info,
+    (GBoxedCopyFunc) bse_sample_file_info_copy_shallow_internal,
+    (GBoxedFreeFunc) bse_sample_file_info_free);
   BSE_TYPE_GCONFIG = sfi_boxed_make_record (&bse_gconfig_boxed_info,
     (GBoxedCopyFunc) bse_gconfig_copy_shallow_internal,
     (GBoxedFreeFunc) bse_gconfig_free);
