@@ -6,10 +6,6 @@
 #define BSE_TYPE_FILE_HANDLER_TYPE	    (BSE_TYPE_ID (BseFileHandlerType))
 extern GType BSE_TYPE_ID (BseFileHandlerType);
 
-/* --- bsebindata.h --- */
-#define BSE_TYPE_BIN_DATA_TYPE	    (BSE_TYPE_ID (BseBinDataType))
-extern GType BSE_TYPE_ID (BseBinDataType);
-
 /* --- bsebiquadfilter.h --- */
 #define BSE_TYPE_BIQUAD_FILTER_TYPE	    (BSE_TYPE_ID (BseBiquadFilterType))
 extern GType BSE_TYPE_ID (BseBiquadFilterType);
@@ -17,8 +13,6 @@ extern GType BSE_TYPE_ID (BseBiquadFilterType);
 extern GType BSE_TYPE_ID (BseBiquadFilterNorm);
 
 /* --- bseenums.h --- */
-#define BSE_TYPE_ENDIAN_TYPE	    (BSE_TYPE_ID (BseEndianType))
-extern GType BSE_TYPE_ID (BseEndianType);
 #define BSE_TYPE_REGISTRATION_TYPE	    (BSE_TYPE_ID (BseRegistrationType))
 extern GType BSE_TYPE_ID (BseRegistrationType);
 #define BSE_TYPE_INTERPOL_TYPE	    (BSE_TYPE_ID (BseInterpolType))
@@ -64,9 +58,6 @@ extern GType BSE_TYPE_ID (BseStandardOscWaveType);
 /* --- bseamplifier.c --- */
 extern GType bse_type_builtin_id_BseAmplifier;
 
-/* --- bsebindata.c --- */
-extern GType bse_type_builtin_id_BseBinData;
-
 /* --- bsebiquadfilter.c --- */
 extern GType bse_type_builtin_id_BseBiquadFilter;
 
@@ -190,12 +181,18 @@ extern GType bse_type_builtin_id_BseWave;
 /* --- bsewaveosc.c --- */
 extern GType bse_type_builtin_id_BseWaveOsc;
 
+/* --- bsecsynth.c --- */
+extern GType bse_type_builtin_id_BseCSynth;
+
 /* --- bsewaverepo.c --- */
 extern GType bse_type_builtin_id_BseWaveRepo;
 
 /* --- bseladspamodule.c --- */
 extern GType bse_type_builtin_id_BseLadspaModule;
 
+/* --- bsepcmwriter.c --- */
+extern GType bse_type_builtin_id_BsePcmWriter;
+
 /**
  ** Generated data ends here
  **/
@@ -220,6 +217,7 @@ typedef struct _BseCategory BseCategory;
 typedef struct _BseDot BseDot;
 typedef struct _BseTrackPart BseTrackPart;
 typedef struct _BseSongTiming BseSongTiming;
+typedef struct _BseSampleFileInfo BseSampleFileInfo;
 typedef struct _BseGConfig BseGConfig;
 
 struct _BseIntSeq {
@@ -310,6 +308,14 @@ struct _BseSongTiming {
   SfiInt tpqn;
   SfiInt tpt;
 };
+struct _BseSampleFileInfo {
+  gchar* file;
+  SfiInt size;
+  SfiNum mtime;
+  gchar* loader;
+  BseStringSeq* waves;
+  SfiInt error;
+};
 struct _BseGConfig {
   SfiInt synth_block_size;
   SfiInt synth_mixing_freq;
@@ -430,6 +436,12 @@ BseSongTiming* bse_song_timing_from_rec (SfiRec *sfi_rec);
 SfiRec *bse_song_timing_to_rec (BseSongTiming* rec);
 void bse_song_timing_free (BseSongTiming* rec);
 
+BseSampleFileInfo* bse_sample_file_info_new (void);
+BseSampleFileInfo* bse_sample_file_info_copy_shallow (BseSampleFileInfo* rec);
+BseSampleFileInfo* bse_sample_file_info_from_rec (SfiRec *sfi_rec);
+SfiRec *bse_sample_file_info_to_rec (BseSampleFileInfo* rec);
+void bse_sample_file_info_free (BseSampleFileInfo* rec);
+
 BseGConfig* bse_gconfig_new (void);
 BseGConfig* bse_gconfig_copy_shallow (BseGConfig* rec);
 BseGConfig* bse_gconfig_from_rec (SfiRec *sfi_rec);
@@ -453,6 +465,8 @@ extern SfiRecFields bse_track_part_fields;
 extern GType BSE_TYPE_TRACK_PART;
 extern SfiRecFields bse_song_timing_fields;
 extern GType BSE_TYPE_SONG_TIMING;
+extern SfiRecFields bse_sample_file_info_fields;
+extern GType BSE_TYPE_SAMPLE_FILE_INFO;
 extern SfiRecFields bse_gconfig_fields;
 extern GType BSE_TYPE_GCONFIG;
 extern GType BSE_TYPE_INT_SEQ;
