@@ -4,10 +4,10 @@
  **/
 #line 1 "bsesnet.proc"
 
-#include        <bse/bseplugin.h>
-#include        <bse/bseprocedure.h>
-#include        <bse/bsesnet.h>
-#include        <bse/bsemain.h>
+#include <bse/bseplugin.h>
+#include <bse/bseprocedure.h>
+#include <bse/bsesnet.h>
+#include <bse/bsemain.h>
 
 
 #line 27 "bsesnet.proc"
@@ -98,7 +98,7 @@ can_create_source_exec (BseProcedureClass *proc,
   
 #line 75 "bsesnet.proc"
   
-  if (!BSE_SNET_USER_SYNTH (snet) && !BSE_DVL_EXT)
+  if (!BSE_SNET_USER_SYNTH (snet) && !BSE_DBG_EXT)
     error = BSE_ERROR_NOT_OWNER;
   else if (!g_type_is_a (type, BSE_TYPE_SOURCE) ||
 	   g_type_is_a (type, BSE_TYPE_CONTAINER))
@@ -141,30 +141,30 @@ create_source_exec (BseProcedureClass *proc,
 {
 #line 100 "bsesnet.proc"
   
-  BseSNet *snet = bse_value_get_object (in_values++);
-  const gchar *type_name = sfi_value_get_string (in_values++);
-  BseContainer *container;
-  BseSource *source;
+  BseContainer *container = bse_value_get_object (in_values++);
+  const gchar *type_name  = sfi_value_get_string (in_values++);
+  BseItem *child;
+  BseUndoStack *ustack;
   BseErrorType error;
-  
+
 #line 107 "bsesnet.proc"
   
-  if (!BSE_IS_SNET (snet))
+  if (!BSE_IS_SNET (container))
     return BSE_ERROR_PROC_PARAM_INVAL;
-  if (bse_item_exec (snet, "can-create-source", type_name, &error) != 0 ||
+  if (bse_item_exec (container, "can-create-source", type_name, &error) != 0 ||
       error != 0)
     return BSE_ERROR_PROC_PARAM_INVAL;
   
 #line 114 "bsesnet.proc"
   
-  container = BSE_CONTAINER (snet);
-  source = g_object_new (g_type_from_name (type_name), NULL);
-  bse_container_add_item (container, BSE_ITEM (source));
+  ustack = bse_item_undo_open (container, "create-source");
+  child = bse_container_new_child (container, g_type_from_name (type_name), NULL);
+  bse_item_push_undo_proc (container, "remove-source", child);
+  bse_item_undo_close (ustack);
+
+#line 120 "bsesnet.proc"
   
-#line 119 "bsesnet.proc"
-  
-  bse_value_set_object (out_values++, source);
-  g_object_unref (source);
+  bse_value_set_object (out_values++, child);
   
   return BSE_ERROR_NONE;
 }
@@ -200,22 +200,35 @@ remove_source_exec (BseProcedureClass *proc,
 {
 #line 139 "bsesnet.proc"
   
-  BseSNet *snet = bse_value_get_object (in_values++);
-  BseSource *source = bse_value_get_object (in_values++);
+  BseSNet *self      = bse_value_get_object (in_values++);
+  BseItem *child     = bse_value_get_object (in_values++);
   BseErrorType error = BSE_ERROR_NONE;
+  BseUndoStack *ustack;
+
+#line 145 "bsesnet.proc"
   
-#line 144 "bsesnet.proc"
-  
-  if (!BSE_IS_SNET (snet) || !BSE_IS_SOURCE (source) || BSE_ITEM (source)->parent != BSE_ITEM (snet) ||
-      (!BSE_SNET_USER_SYNTH (snet) && !BSE_DVL_EXT))
+  if (!BSE_IS_SNET (self) || !BSE_IS_SOURCE (child) || child->parent != (BseItem*) self ||
+      (!BSE_SNET_USER_SYNTH (self) && !BSE_DBG_EXT))
     return BSE_ERROR_PROC_PARAM_INVAL;
   
-#line 149 "bsesnet.proc"
-  
-  bse_container_remove_item (BSE_CONTAINER (snet), BSE_ITEM (source));
+#line 150 "bsesnet.proc"
   
+  ustack = bse_item_undo_open (self, "remove-child %s", bse_object_debug_name (child));
 #line 152 "bsesnet.proc"
   
+  bse_container_uncross_undoable (BSE_CONTAINER (self), child);
+#line 154 "bsesnet.proc"
+  
+  bse_item_push_redo_proc (self, "remove-source", child);
+#line 156 "bsesnet.proc"
+  
+  bse_container_remove_backedup (BSE_CONTAINER (self), child, ustack);
+#line 158 "bsesnet.proc"
+  
+  bse_item_undo_close (ustack);
+  
+#line 161 "bsesnet.proc"
+  
   g_value_set_enum (out_values++, error);
   
   return BSE_ERROR_NONE;
