@@ -1,5 +1,5 @@
 /* BSE - Bedevilled Sound Engine
- * Copyright (C) 2002 Tim Janik
+ * Copyright (C) 2002-2003 Tim Janik
  *
  * This library is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,11 +20,7 @@
 
 #include	<bse/bseitem.h>
 
-
-#ifdef __cplusplus
-extern "C" {
-#endif /* __cplusplus */
-
+G_BEGIN_DECLS
 
 /* --- BSE type macros --- */
 #define BSE_TYPE_TRACK		    (BSE_TYPE_ID (BseTrack))
@@ -36,20 +32,34 @@ extern "C" {
 
 
 /* --- BseTrack --- */
+typedef struct {
+  guint    tick;
+  guint    id;
+  BsePart *part;
+} BseTrackEntry;
 struct _BseTrack
 {
   BseItem	   parent_instance;
 
-  BsePart	  *part_SL;
-  BseSNet	  *snet;
   guint		   max_voices;
+  BseSNet	  *snet;
 
-  BseMidiReceiver *midi_receiver_SL;
+  /* wave synthesis */
+  BseWave	  *wave;
+  BseSNet	  *wnet;
 
-  BseSource       *voice_input;
+  /* playback intergration */
   BseSource       *sub_synth;
+  BseSource       *voice_input;
   BseSource       *voice_switch;
   BseSource       *context_merger;
+
+  /* fields protected by sequencer mutex */
+  guint		   n_entries_SL : 30;
+  guint		   muted_SL : 1;
+  BseTrackEntry	  *entries_SL;
+  BseMidiReceiver *midi_receiver_SL;
+  gboolean	   track_done_SL;
 };
 struct _BseTrackClass
 {
@@ -67,10 +77,24 @@ void	bse_track_clone_voices		(BseTrack		*self,
 					 BseSNet		*snet,
 					 guint			 context,
 					 GslTrans		*trans);
+guint        	 bse_track_insert_part	(BseTrack		*self,
+					 guint			 tick,
+					 BsePart		*part);
+void		 bse_track_remove_tick	(BseTrack		*self,
+					 guint			 tick);
+BseTrackPartSeq* bse_track_list_parts	(BseTrack		*self);
+gboolean	 bse_track_find_part	(BseTrack		*self,
+					 BsePart		*part,
+					 guint			*start_p);
+BseTrackEntry*	 bse_track_lookup_tick	(BseTrack		*self,
+					 guint			 tick);
+BseTrackEntry*   bse_track_find_link    (BseTrack               *self,
+                                         guint                   id);
+BsePart*	 bse_track_get_part_SL	(BseTrack		*self,
+					 guint			 tick,
+					 guint			*start,
+					 guint			*next);
 
-
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
+G_END_DECLS
 
 #endif /* __BSE_TRACK_H__ */
