@@ -1,5 +1,5 @@
 /* GSL - Generic Sound Layer
- * Copyright (C) 2001-2002 Tim Janik and Stefan Westerfeld
+ * Copyright (C) 2001-2003 Tim Janik and Stefan Westerfeld
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -21,10 +21,7 @@
 
 #include <bse/gsldefs.h>
 
-#ifdef __cplusplus
-extern "C" {
-#endif /* __cplusplus */
-
+G_BEGIN_DECLS
 
 
 /* --- global initialization --- */
@@ -61,7 +58,7 @@ guint64		   gsl_tick_stamp	(void);
 guint64		   gsl_time_system	(void);
 GslTickStampUpdate gsl_tick_stamp_last	(void);
 #define		   GSL_TICK_STAMP	(_GSL_TICK_STAMP_VAL ())
-#define		   GSL_MAX_TICK_STAMP	(G_MAXUINT64)
+#define		   GSL_MAX_TICK_STAMP	(18446744073709551615LLU /* 2^64-1*/)
 void		gsl_thread_awake_before	(guint64	 tick_stamp);
 #define	GSL_SPIN_LOCK	SFI_SPIN_LOCK
 #define	GSL_SPIN_UNLOCK	SFI_SPIN_UNLOCK
@@ -79,6 +76,33 @@ GslErrorType gsl_check_file		(const gchar	*file_name,
 					 const gchar	*mode);
 
 
+/* --- progress notification --- */
+typedef struct _GslProgressState GslProgressState;
+typedef guint (*GslProgressFunc)        (gpointer          data,
+                                         gfloat            pval, /* -1, 0..100 */
+                                         const gchar      *detail,
+                                         GslProgressState *pstate);
+struct _GslProgressState
+{
+  guint           wipe_length, precision;
+  gfloat          pval, epsilon;
+  gpointer        pdata;
+  GslProgressFunc pfunc;
+};
+GslProgressState gsl_progress_state     (gpointer          data,
+                                         GslProgressFunc   pfunc,
+                                         guint             precision);
+void             gsl_progress_notify    (GslProgressState *pstate,
+                                         gfloat            pval,
+                                         const gchar      *detail_format,
+                                         ...);
+void             gsl_progress_wipe      (GslProgressState *pstate);
+guint            gsl_progress_printerr  (gpointer          message,
+                                         gfloat            pval,
+                                         const gchar      *detail,
+                                         GslProgressState *pstate);
+
+
 /* --- implementation details --- */
 void	       _gsl_tick_stamp_inc	(void);
 void	       _gsl_tick_stamp_set_leap (guint		 ticks);
@@ -95,8 +119,6 @@ void	_gsl_init_loader_mad		(void);
 extern volatile guint64	gsl_externvar_tick_stamp;
 
 
-#ifdef __cplusplus
-}
-#endif /* __cplusplus */
+G_END_DECLS
 
 #endif /* __GSL_COMMON_H__ */
