@@ -136,8 +136,9 @@ oggv_create_chunk_handle (gpointer      data,
 
   g_return_val_if_fail (nth_chunk == 0, NULL);
 
-  dhandle = gsl_data_handle_new_ogg_vorbis (fi->wfi.file_name,
-					    wdsc->chunks[0].loader_offset); /* lbitstream */
+  dhandle = gsl_data_handle_new_ogg_vorbis_muxed (fi->wfi.file_name,
+					          wdsc->chunks[0].loader_offset, /* lbitstream */
+					          wdsc->chunks[0].osc_freq);
   if (!dhandle)
     *error_p = GSL_ERROR_OPEN_FAILED;
   return dhandle;
