@@ -90,6 +90,7 @@ bse_int_seq_free (BseIntSeq* seq)
 {
   g_return_if_fail (seq != NULL);
   
+  g_free (seq->ints);
   g_free (seq);
 }
 
@@ -179,6 +180,7 @@ bse_note_seq_free (BseNoteSeq* seq)
 {
   g_return_if_fail (seq != NULL);
   
+  g_free (seq->notes);
   g_free (seq);
 }
 
@@ -278,6 +280,7 @@ bse_string_seq_free (BseStringSeq* seq)
   
   for (i = 0; i < seq->n_strings; i++)
     g_free (seq->strings[i]);
+  g_free (seq->strings);
   g_free (seq);
 }
 
@@ -367,6 +370,7 @@ bse_proxy_seq_free (BseProxySeq* seq)
 {
   g_return_if_fail (seq != NULL);
   
+  g_free (seq->proxies);
   g_free (seq);
 }
 
@@ -470,6 +474,7 @@ bse_part_note_seq_free (BsePartNoteSeq* seq)
   
   for (i = 0; i < seq->n_pnotes; i++)
     bse_part_note_free (seq->pnotes[i]);
+  g_free (seq->pnotes);
   g_free (seq);
 }
 
@@ -573,6 +578,7 @@ bse_category_seq_free (BseCategorySeq* seq)
   
   for (i = 0; i < seq->n_cats; i++)
     bse_category_free (seq->cats[i]);
+  g_free (seq->cats);
   g_free (seq);
 }
 
@@ -676,6 +682,7 @@ bse_dot_seq_free (BseDotSeq* seq)
   
   for (i = 0; i < seq->n_dots; i++)
     bse_dot_free (seq->dots[i]);
+  g_free (seq->dots);
   g_free (seq);
 }
 
@@ -779,6 +786,7 @@ bse_track_part_seq_free (BseTrackPartSeq* seq)
   
   for (i = 0; i < seq->n_tparts; i++)
     bse_track_part_free (seq->tparts[i]);
+  g_free (seq->tparts);
   g_free (seq);
 }
 
@@ -793,10 +801,10 @@ bse_part_note_new (void)
 BsePartNote*
 bse_part_note_copy_shallow (BsePartNote* rec)
 {
-  BsePartNote* rec_copy = NULL;
+  BsePartNote* rec_copy;
 
   g_return_val_if_fail (rec != NULL, NULL);
-  rec_copy = bse_part_note_new ();
+  rec_copy = g_new0 (BsePartNote, 1);
   rec_copy->id = rec->id;
   rec_copy->tick = rec->tick;
   rec_copy->duration = rec->duration;
@@ -892,10 +900,10 @@ bse_note_description_new (void)
 BseNoteDescription*
 bse_note_description_copy_shallow (BseNoteDescription* rec)
 {
-  BseNoteDescription* rec_copy = NULL;
+  BseNoteDescription* rec_copy;
 
   g_return_val_if_fail (rec != NULL, NULL);
-  rec_copy = bse_note_description_new ();
+  rec_copy = g_new0 (BseNoteDescription, 1);
   rec_copy->note = rec->note;
   rec_copy->octave = rec->octave;
   rec_copy->freq = rec->freq;
@@ -1014,10 +1022,10 @@ bse_note_sequence_new (void)
 BseNoteSequence*
 bse_note_sequence_copy_shallow (BseNoteSequence* rec)
 {
-  BseNoteSequence* rec_copy = NULL;
+  BseNoteSequence* rec_copy;
 
   g_return_val_if_fail (rec != NULL, NULL);
-  rec_copy = bse_note_sequence_new ();
+  rec_copy = g_new0 (BseNoteSequence, 1);
   rec_copy->offset = rec->offset;
   rec_copy->notes = bse_note_seq_copy_shallow (rec->notes);
   return rec_copy;
@@ -1082,10 +1090,10 @@ bse_icon_new (void)
 BseIcon*
 bse_icon_copy_shallow (BseIcon* rec)
 {
-  BseIcon* rec_copy = NULL;
+  BseIcon* rec_copy;
 
   g_return_val_if_fail (rec != NULL, NULL);
-  rec_copy = bse_icon_new ();
+  rec_copy = g_new0 (BseIcon, 1);
   rec_copy->bytes_per_pixel = rec->bytes_per_pixel;
   rec_copy->width = rec->width;
   rec_copy->height = rec->height;
@@ -1164,10 +1172,10 @@ bse_category_new (void)
 BseCategory*
 bse_category_copy_shallow (BseCategory* rec)
 {
-  BseCategory* rec_copy = NULL;
+  BseCategory* rec_copy;
 
   g_return_val_if_fail (rec != NULL, NULL);
-  rec_copy = bse_category_new ();
+  rec_copy = g_new0 (BseCategory, 1);
   rec_copy->category_id = rec->category_id;
   rec_copy->category = g_strdup (rec->category);
   rec_copy->mindex = rec->mindex;
@@ -1261,10 +1269,10 @@ bse_dot_new (void)
 BseDot*
 bse_dot_copy_shallow (BseDot* rec)
 {
-  BseDot* rec_copy = NULL;
+  BseDot* rec_copy;
 
   g_return_val_if_fail (rec != NULL, NULL);
-  rec_copy = bse_dot_new ();
+  rec_copy = g_new0 (BseDot, 1);
   rec_copy->x = rec->x;
   rec_copy->y = rec->y;
   return rec_copy;
@@ -1325,10 +1333,10 @@ bse_track_part_new (void)
 BseTrackPart*
 bse_track_part_copy_shallow (BseTrackPart* rec)
 {
-  BseTrackPart* rec_copy = NULL;
+  BseTrackPart* rec_copy;
 
   g_return_val_if_fail (rec != NULL, NULL);
-  rec_copy = bse_track_part_new ();
+  rec_copy = g_new0 (BseTrackPart, 1);
   rec_copy->tick = rec->tick;
   rec_copy->part = rec->part;
   rec_copy->duration = rec->duration;
@@ -1396,10 +1404,10 @@ bse_song_timing_new (void)
 BseSongTiming*
 bse_song_timing_copy_shallow (BseSongTiming* rec)
 {
-  BseSongTiming* rec_copy = NULL;
+  BseSongTiming* rec_copy;
 
   g_return_val_if_fail (rec != NULL, NULL);
-  rec_copy = bse_song_timing_new ();
+  rec_copy = g_new0 (BseSongTiming, 1);
   rec_copy->tick = rec->tick;
   rec_copy->bpm = rec->bpm;
   rec_copy->nominator = rec->nominator;
@@ -1489,10 +1497,10 @@ bse_sample_file_info_new (void)
 BseSampleFileInfo*
 bse_sample_file_info_copy_shallow (BseSampleFileInfo* rec)
 {
-  BseSampleFileInfo* rec_copy = NULL;
+  BseSampleFileInfo* rec_copy;
 
   g_return_val_if_fail (rec != NULL, NULL);
-  rec_copy = bse_sample_file_info_new ();
+  rec_copy = g_new0 (BseSampleFileInfo, 1);
   rec_copy->file = g_strdup (rec->file);
   rec_copy->size = rec->size;
   rec_copy->mtime = rec->mtime;
@@ -1586,10 +1594,10 @@ bse_gconfig_new (void)
 BseGConfig*
 bse_gconfig_copy_shallow (BseGConfig* rec)
 {
-  BseGConfig* rec_copy = NULL;
+  BseGConfig* rec_copy;
 
   g_return_val_if_fail (rec != NULL, NULL);
-  rec_copy = bse_gconfig_new ();
+  rec_copy = g_new0 (BseGConfig, 1);
   rec_copy->synth_block_size = rec->synth_block_size;
   rec_copy->synth_mixing_freq = rec->synth_mixing_freq;
   rec_copy->invert_sustain = rec->invert_sustain;
@@ -2472,8 +2480,13 @@ bse_server_preferences_locked (SfiProxy bse_server_object) {
 }
 
 void
-bse_server_register_plugins (SfiProxy bse_server_object) {
-  sfi_glue_vcall_void ("BseServer+register-plugins", 'p', bse_server_object, 0);
+bse_server_register_core_plugins (SfiProxy bse_server_object) {
+  sfi_glue_vcall_void ("BseServer+register-core-plugins", 'p', bse_server_object, 0);
+}
+
+void
+bse_server_register_ladspa_plugins (SfiProxy bse_server_object) {
+  sfi_glue_vcall_void ("BseServer+register-ladspa-plugins", 'p', bse_server_object, 0);
 }
 
 void
