@@ -218,7 +218,8 @@
 #define bse_server_note_from_freq foo_prefix_bse_server_note_from_freq
 #define bse_server_note_from_string foo_prefix_bse_server_note_from_string
 #define bse_server_preferences_locked foo_prefix_bse_server_preferences_locked
-#define bse_server_register_plugins foo_prefix_bse_server_register_plugins
+#define bse_server_register_core_plugins foo_prefix_bse_server_register_core_plugins
+#define bse_server_register_ladspa_plugins foo_prefix_bse_server_register_ladspa_plugins
 #define bse_server_register_scripts foo_prefix_bse_server_register_scripts
 #define bse_server_save_preferences foo_prefix_bse_server_save_preferences
 #define bse_server_use_new_project foo_prefix_bse_server_use_new_project
@@ -793,7 +794,8 @@ SfiInt bse_server_n_scripts (SfiProxy bse_server_object);
 BseNoteDescription* bse_server_note_from_freq (SfiProxy bse_server_object, SfiReal freq);
 BseNoteDescription* bse_server_note_from_string (SfiProxy bse_server_object, const gchar* name);
 SfiBool bse_server_preferences_locked (SfiProxy bse_server_object);
-void bse_server_register_plugins (SfiProxy bse_server_object);
+void bse_server_register_core_plugins (SfiProxy bse_server_object);
+void bse_server_register_ladspa_plugins (SfiProxy bse_server_object);
 void bse_server_register_scripts (SfiProxy bse_server_object);
 void bse_server_save_preferences (SfiProxy bse_server_object);
 SfiProxy bse_server_use_new_project (SfiProxy bse_server_object, const gchar* name);
