@@ -19,6 +19,8 @@
 #include <sfi/sfi.h>
 #include <unistd.h>
 #include <string.h>
+#include <signal.h>	/* G_BREAKPOINT() */
+
 
 /* provide IDL type initializers */
 #define sfidl_pspec_Real(group, name, nick, blurb, dflt, min, max, step, hints)  \
@@ -40,7 +42,7 @@
 #define	TICK()		do g_print ("-"); while (0)
 #define	XTICK()		do g_print ("X"); while (0)
 #define	DONE()		do g_print ("]\n"); while (0)
-#define	ASSERT(code)	do { if (code) TICK (); else g_error ("failed to assert: %s", G_STRINGIFY (code)); } while (0)
+#define	ASSERT(code)	do { if (code) TICK (); else g_error ("(line:%u) failed to assert: %s", __LINE__, #code); } while (0)
 
 static void
 test_misc (void)
@@ -79,6 +81,9 @@ test_time (void)
   ASSERT (sfi_time_from_string_err (str, &error) == t);
   ASSERT (error == NULL);
   g_free (str);
+  /* test hard boundaries */
+  ASSERT (sfi_time_from_string ("1990-01-01 00:00:00") == SFI_MIN_TIME);
+  ASSERT (sfi_time_from_string ("2038-01-19 03:14:07") == SFI_MAX_TIME);
   /* test error returns */
   ASSERT (sfi_time_from_string_err ("foo 22", &error) == 0);
   ASSERT (error != NULL);
@@ -90,7 +95,7 @@ test_time (void)
       if (!error)
 	TICK ();
       else
-	g_print ("{failed to parse \"%s\": %s\n}", time_strings[i], error);
+	g_print ("{failed to parse \"%s\": %s (got: %s)\n}", time_strings[i], error, sfi_time_to_string (t)); /* memleak */
       g_free (error);
       str = sfi_time_to_string (t);
       ASSERT (sfi_time_from_string_err (str, &error) == t);
@@ -199,18 +204,18 @@ test_scanner64 (void)
   SCANNER_ASSERT64 (scanner, FALSE, G_TOKEN_BINARY, " 0b0 #", 0);
   SCANNER_ASSERT64 (scanner, FALSE, G_TOKEN_BINARY, " 0b10000000000000000 #", 65536);
   SCANNER_ASSERT64 (scanner, FALSE, G_TOKEN_BINARY, " 0b11111111111111111111111111111111 #", 4294967295U);
-  SCANNER_ASSERT64 (scanner, FALSE, G_TOKEN_BINARY, " 0b1111111111111111111111111111111111111111111111111111111111111111 #", 18446744073709551615U);
+  SCANNER_ASSERT64 (scanner, FALSE, G_TOKEN_BINARY, " 0b1111111111111111111111111111111111111111111111111111111111111111 #", 18446744073709551615ULL);
   SCANNER_ASSERT64 (scanner, FALSE, G_TOKEN_OCTAL, " 0 #", 0);
   SCANNER_ASSERT64 (scanner, FALSE, G_TOKEN_OCTAL, " 0200000 #", 65536);
   SCANNER_ASSERT64 (scanner, FALSE, G_TOKEN_OCTAL, " 037777777777 #", 4294967295U);
-  SCANNER_ASSERT64 (scanner, FALSE, G_TOKEN_OCTAL, " 01777777777777777777777 #", 18446744073709551615U);
+  SCANNER_ASSERT64 (scanner, FALSE, G_TOKEN_OCTAL, " 01777777777777777777777 #", 18446744073709551615ULL);
   SCANNER_ASSERT64 (scanner, FALSE, G_TOKEN_HEX, " 0x0 #", 0);
   SCANNER_ASSERT64 (scanner, FALSE, G_TOKEN_HEX, " 0x10000 #", 65536);
   SCANNER_ASSERT64 (scanner, FALSE, G_TOKEN_HEX, " 0xffffffff #", 4294967295U);
-  SCANNER_ASSERT64 (scanner, FALSE, G_TOKEN_HEX, " 0xffffffffffffffff #", 18446744073709551615U);
+  SCANNER_ASSERT64 (scanner, FALSE, G_TOKEN_HEX, " 0xffffffffffffffff #", 18446744073709551615ULL);
   SCANNER_ASSERT64 (scanner, FALSE, G_TOKEN_INT, " 65536 #", 65536);
   SCANNER_ASSERT64 (scanner, FALSE, G_TOKEN_INT, " 4294967295 #", 4294967295U);
-  SCANNER_ASSERT64 (scanner, FALSE, G_TOKEN_INT, " 18446744073709551615 #", 18446744073709551615U);
+  SCANNER_ASSERT64 (scanner, FALSE, G_TOKEN_INT, " 18446744073709551615 #", 18446744073709551615ULL);
   SCANNER_ASSERTf (scanner, FALSE, G_TOKEN_FLOAT, " 0.0 #", 0);
   SCANNER_ASSERTf (scanner, FALSE, G_TOKEN_FLOAT, " 2.2250738585072014e-308 #", 2.2250738585072014e-308);
   SCANNER_ASSERTf (scanner, FALSE, G_TOKEN_FLOAT, " 1.7976931348623157e+308 #", 1.7976931348623157e+308);
@@ -313,7 +318,7 @@ serialize_cmp (GValue     *value,
 	    }
 	}
       ASSERT (cmp == 0);
-      if (1) /* generate testoutput */
+      if (0) /* FIXME: generate testoutput */
 	g_print ("OK=================(%s)=================:\n%s\n", pspec->name, gstring->str);
     }
   g_scanner_destroy (scanner);
@@ -669,7 +674,7 @@ test_vmarshal_func4 (gpointer o,
 {
   ASSERT (o == pointer1);
   ASSERT (r == -426.9112e-267);
-  ASSERT (n == -2598768763298128732);
+  ASSERT (n == -2598768763298128732LL);
   ASSERT (data == pointer3);
 }
 
@@ -684,7 +689,7 @@ test_vmarshal_func7 (gpointer o,
 {
   ASSERT (o == pointer1);
   ASSERT (r == -426.9112e-267);
-  ASSERT (n == -2598768763298128732);
+  ASSERT (n == -2598768763298128732LL);
   ASSERT (p == (SfiProxy) pointer2);
   ASSERT (i == -2134567);
   ASSERT (self == (SfiNum) test_vmarshal_func7);
@@ -697,7 +702,7 @@ test_vmarshals (void)
   SfiSeq *seq = sfi_seq_new ();
   MSG ("Vmarshals:");
   sfi_seq_append_real (seq, -426.9112e-267);
-  sfi_seq_append_num (seq, -2598768763298128732);
+  sfi_seq_append_num (seq, -2598768763298128732LL);
   sfi_vmarshal_void (test_vmarshal_func4, pointer1,
 		  seq->n_elements, seq->elements,
 		  pointer3);
