@@ -1,5 +1,5 @@
 /* BSE-SCM - Bedevilled Sound Engine Scheme Wrapper
- * Copyright (C) 2002 Tim Janik
+ * Copyright (C) 2002-2003 Tim Janik
  *
  * This library is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,7 +16,6 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
  */
 #define G_LOG_DOMAIN "BseShell"
-
 #include <string.h>
 #include <errno.h>
 #include "bsescminterp.h"
@@ -47,17 +46,19 @@
 
 /* --- prototypes --- */
 static SCM	bse_scm_from_value		(const GValue	*value);
-
+static GValue*	bse_value_from_scm		(SCM		 sval);
 
 
 /* --- SCM GC hooks --- */
+static gulong tc_glue_gc_cell = 0;
+#define SCM_IS_GLUE_GC_CELL(sval)    (SCM_SMOB_PREDICATE (tc_glue_gc_cell, sval))
+#define SCM_GET_GLUE_GC_CELL(sval)   ((BseScmGCCell*) SCM_SMOB_DATA (sval))
 typedef void (*BseScmFreeFunc) ();
 typedef struct {
   gpointer       data;
   BseScmFreeFunc free_func;
   gsize          size_hint;
 } BseScmGCCell;
-static gulong tc_gc_cell = 0;
 static void
 bse_scm_enter_gc (SCM           *scm_gc_list,
 		  gpointer       data,
@@ -77,7 +78,7 @@ bse_scm_enter_gc (SCM           *scm_gc_list,
   gc_cell->free_func = free_func;
   gc_cell->size_hint = size_hint + sizeof (BseScmGCCell);
 
-  SCM_NEWSMOB (s_cell, tc_gc_cell, gc_cell);
+  SCM_NEWSMOB (s_cell, tc_glue_gc_cell, gc_cell);
   *scm_gc_list = gh_cons (s_cell, *scm_gc_list);
   scm_done_malloc (gc_cell->size_hint);
 }
@@ -97,7 +98,7 @@ bse_scm_mark_gc_cell (SCM scm_gc_cell)
 static scm_sizet
 bse_scm_free_gc_cell (SCM scm_gc_cell)
 {
-  BseScmGCCell *gc_cell = (BseScmGCCell*) SCM_CDR (scm_gc_cell);
+  BseScmGCCell *gc_cell = SCM_GET_GLUE_GC_CELL (scm_gc_cell);
   scm_sizet size = gc_cell->size_hint;
 
   // g_printerr ("GCCell freeing %u bytes (%p).\n", size, gc_cell->free_func);
@@ -111,6 +112,8 @@ bse_scm_free_gc_cell (SCM scm_gc_cell)
 
 /* --- SCM Glue GC Plateau --- */
 static gulong tc_glue_gc_plateau = 0;
+#define SCM_IS_GLUE_GC_PLATEAU(sval)    (SCM_SMOB_PREDICATE (tc_glue_gc_plateau, sval))
+#define SCM_GET_GLUE_GC_PLATEAU(sval)   ((GcPlateau*) SCM_SMOB_DATA (sval))
 static guint  scm_glue_gc_plateau_blocker = 0;
 typedef struct {
   guint    size_hint;
@@ -136,9 +139,9 @@ bse_scm_destroy_gc_plateau (SCM s_gcplateau)
 {
   GcPlateau *gp;
 
-  g_assert (SCM_NIMP (s_gcplateau) && SCM_CAR (s_gcplateau) == tc_glue_gc_plateau);
+  g_assert (SCM_IS_GLUE_GC_PLATEAU (s_gcplateau));
 
-  gp = (GcPlateau*) SCM_CDR (s_gcplateau);
+  gp = SCM_GET_GLUE_GC_PLATEAU (s_gcplateau);
   if (gp->active_plateau)
     {
       gp->active_plateau = FALSE;
@@ -152,7 +155,7 @@ bse_scm_destroy_gc_plateau (SCM s_gcplateau)
 static scm_sizet
 bse_scm_gc_plateau_free (SCM s_gcplateau)
 {
-  GcPlateau *gp = (GcPlateau*) SCM_CDR (s_gcplateau);
+  GcPlateau *gp = SCM_GET_GLUE_GC_PLATEAU (s_gcplateau);
   guint size_hint = gp->size_hint;
 
   bse_scm_destroy_gc_plateau (s_gcplateau);
@@ -164,8 +167,10 @@ bse_scm_gc_plateau_free (SCM s_gcplateau)
 
 /* --- SCM Glue Record --- */
 static gulong tc_glue_rec = 0;
+#define SCM_IS_GLUE_REC(sval)    (SCM_SMOB_PREDICATE (tc_glue_rec, sval))
+#define SCM_GET_GLUE_REC(sval)   ((SfiRec*) SCM_SMOB_DATA (sval))
 static SCM
-bse_scm_make_glue_rec (SfiRec *rec)
+bse_scm_from_glue_rec (SfiRec *rec)
 {
   SCM s_rec = 0;
 
@@ -176,10 +181,33 @@ bse_scm_make_glue_rec (SfiRec *rec)
   return s_rec;
 }
 
+static SCM
+bse_scm_glue_rec_new (SCM sfields)
+{
+  SfiRec *rec;
+  SCM s_rec = 0;
+  if (!SCM_UNBNDP (sfields))
+    SCM_ASSERT (SCM_CONSP (sfields) || SCM_EQ_P (sfields, SCM_EOL), sfields, SCM_ARG1, "bse-rec-new");
+  rec = sfi_rec_new ();
+  s_rec = bse_scm_from_glue_rec (rec);
+  sfi_rec_unref (rec);
+  if (!SCM_UNBNDP (sfields))
+    {
+      SCM node;
+      for (node = sfields; SCM_CONSP (node); node = SCM_CDR (node))
+        {
+          SCM scons = SCM_CAR (node);
+          SCM_ASSERT (SCM_CONSP (scons), sfields, SCM_ARG1, "bse-rec-new");
+          bse_scm_glue_rec_set (s_rec, SCM_CAR (scons), SCM_CDR (scons));
+        }
+    }
+  return s_rec;
+}
+
 static scm_sizet
 bse_scm_free_glue_rec (SCM scm_rec)
 {
-  SfiRec *rec = (SfiRec*) SCM_CDR (scm_rec);
+  SfiRec *rec = SCM_GET_GLUE_REC (scm_rec);
 
   sfi_rec_unref (rec);
   return 0;
@@ -192,15 +220,17 @@ bse_scm_glue_rec_print (SCM scm_rec)
   SfiRec *rec;
   guint i;
 
-  SCM_ASSERT ((SCM_NIMP (scm_rec) && SCM_CAR (scm_rec) == tc_glue_rec), scm_rec, SCM_ARG1, "bse-record-print");
+  SCM_ASSERT (SCM_IS_GLUE_REC (scm_rec), scm_rec, SCM_ARG1, "bse-rec-print");
 
-  rec = (SfiRec*) SCM_CDR (scm_rec);
+  rec = SCM_GET_GLUE_REC (scm_rec);
   scm_puts ("'(", port);
   for (i = 0; i < rec->n_fields; i++)
     {
       const gchar *name = rec->field_names[i];
       GValue *value = rec->fields + i;
 
+      if (i)
+        scm_puts (" ", port);
       scm_puts ("(", port);
       scm_puts (name, port);
       scm_puts (" . ", port);
@@ -222,10 +252,10 @@ bse_scm_glue_rec_get (SCM scm_rec,
   gchar *name;
   SCM s_val;
 
-  SCM_ASSERT ((SCM_NIMP (scm_rec) && SCM_CAR (scm_rec) == tc_glue_rec), scm_rec, SCM_ARG1, "bse-record-get");
-  SCM_ASSERT (SCM_SYMBOLP (s_field),  s_field,  SCM_ARG2, "bse-record-get");
+  SCM_ASSERT (SCM_IS_GLUE_REC (scm_rec), scm_rec, SCM_ARG1, "bse-rec-get");
+  SCM_ASSERT (SCM_SYMBOLP (s_field),  s_field,  SCM_ARG2, "bse-rec-get");
 
-  rec = (SfiRec*) SCM_CDR (scm_rec);
+  rec = SCM_GET_GLUE_REC (scm_rec);
   name = g_strndup (SCM_ROCHARS (s_field), SCM_LENGTH (s_field));
   val = sfi_rec_get (rec, name);
   g_free (name);
@@ -238,9 +268,35 @@ bse_scm_glue_rec_get (SCM scm_rec,
   return s_val;
 }
 
+SCM
+bse_scm_glue_rec_set (SCM scm_rec,
+		      SCM s_field,
+                      SCM s_value)
+{
+  SCM gcplateau = bse_scm_make_gc_plateau (1024);
+  GValue *val;
+  SfiRec *rec;
+  gchar *name;
+
+  SCM_ASSERT (SCM_IS_GLUE_REC (scm_rec), scm_rec, SCM_ARG1, "bse-rec-set");
+  SCM_ASSERT (SCM_SYMBOLP (s_field),  s_field,  SCM_ARG2, "bse-rec-set");
+
+  rec = SCM_GET_GLUE_REC (scm_rec);
+  val = bse_value_from_scm (s_value);
+  if (!val)
+    SCM_ASSERT (FALSE, s_value, SCM_ARG3, "bse-rec-set");
+  name = g_strndup (SCM_ROCHARS (s_field), SCM_LENGTH (s_field));
+  sfi_rec_set (rec, name, val);
+  g_free (name);
+  bse_scm_destroy_gc_plateau (gcplateau);
+  return SCM_UNSPECIFIED;
+}
+
 
 /* --- SCM Glue Proxy --- */
 static gulong tc_glue_proxy = 0;
+#define SCM_IS_GLUE_PROXY(sval)    (SCM_SMOB_PREDICATE (tc_glue_proxy, sval))
+#define SCM_GET_GLUE_PROXY(sval)   ((SfiProxy) SCM_SMOB_DATA (sval))
 static SCM    glue_null_proxy;
 static SCM
 bse_scm_make_glue_proxy (SfiProxy proxy)
@@ -256,8 +312,8 @@ static SCM
 bse_scm_proxy_equalp (SCM scm_p1,
 		      SCM scm_p2)
 {
-  SfiProxy p1 = SCM_CDR (scm_p1);
-  SfiProxy p2 = SCM_CDR (scm_p2);
+  SfiProxy p1 = SCM_GET_GLUE_PROXY (scm_p1);
+  SfiProxy p2 = SCM_GET_GLUE_PROXY (scm_p2);
   
   return SCM_BOOL (p1 == p2);
 }
@@ -265,9 +321,9 @@ bse_scm_proxy_equalp (SCM scm_p1,
 static SCM
 bse_scm_null_proxyp (SCM scm_proxy)
 {
-  if (SCM_NIMP (scm_proxy) && SCM_CAR (scm_proxy) == tc_glue_proxy)
+  if (SCM_IS_GLUE_PROXY (scm_proxy))
     {
-      SfiProxy p = SCM_CDR (scm_proxy);
+      SfiProxy p = SCM_GET_GLUE_PROXY (scm_proxy);
       return SCM_BOOL (p == 0);
     }
   return SCM_BOOL_F;
@@ -304,11 +360,11 @@ bse_value_from_scm (SCM sval)
   if (SCM_BOOLP (sval))
     value = sfi_value_bool (!SCM_FALSEP (sval));
   else if (SCM_INUMP (sval))
-    value = sfi_value_int (scm_num2long (sval, (char*) SCM_ARG1, "bse_value_from_scm"));
+    value = sfi_value_int (scm_num2long (sval, 1, "bse_value_from_scm"));
   else if (SCM_REALP (sval))
     value = sfi_value_real (scm_num2dbl (sval, "bse_value_from_scm"));
   else if (SCM_BIGP (sval))
-    value = sfi_value_num (scm_num2long_long (sval, (char*) SCM_ARG1, "bse_value_from_scm"));
+    value = sfi_value_num (scm_num2long_long (sval, 1, "bse_value_from_scm"));
   else if (SCM_SYMBOLP (sval))
     value = sfi_value_lchoice (SCM_ROCHARS (sval), SCM_LENGTH (sval));
   else if (SCM_ROSTRINGP (sval))
@@ -326,14 +382,14 @@ bse_value_from_scm (SCM sval)
       value = sfi_value_seq (seq);
       sfi_seq_unref (seq);
     }
-  else if (SCM_NIMP (sval) && SCM_CAR (sval) == tc_glue_proxy)
+  else if (SCM_IS_GLUE_PROXY (sval))
     {
-      SfiProxy proxy = (SfiProxy) SCM_CDR (sval);
+      SfiProxy proxy = SCM_GET_GLUE_PROXY (sval);
       value = sfi_value_proxy (proxy);
     }
-  else if (SCM_NIMP (sval) && SCM_CAR (sval) == tc_glue_rec)
+  else if (SCM_IS_GLUE_REC (sval))
     {
-      SfiRec *rec = (SfiRec*) SCM_CDR (sval);
+      SfiRec *rec = SCM_GET_GLUE_REC (sval);
       value = sfi_value_rec (rec);
     }
   else
@@ -358,7 +414,7 @@ bse_scm_from_value (const GValue *value)
       sval = gh_long2scm (sfi_value_get_int (value));
       break;
     case SFI_SCAT_NUM:
-      sval = scm_long_long2big (sfi_value_get_num (value));
+      sval = scm_long_long2num (sfi_value_get_num (value));
       break;
     case SFI_SCAT_REAL:
       sval = gh_double2scm (sfi_value_get_real (value));
@@ -369,7 +425,7 @@ bse_scm_from_value (const GValue *value)
       break;
     case SFI_SCAT_CHOICE:
       str = sfi_value_get_choice (value);
-      sval = str ? SCM_CAR (scm_intern0 (str)) : BSE_SCM_NIL;
+      sval = str ? gh_symbol2scm (str) : BSE_SCM_NIL;
       break;
     case SFI_SCAT_BBLOCK:
       sval = BSE_SCM_NIL;
@@ -395,7 +451,7 @@ bse_scm_from_value (const GValue *value)
     case SFI_SCAT_REC:
       rec = sfi_value_get_rec (value);
       if (rec)
-	sval = bse_scm_make_glue_rec (rec);
+	sval = bse_scm_from_glue_rec (rec);
       else
 	sval = BSE_SCM_NIL;
       break;
@@ -418,12 +474,12 @@ bse_scm_glue_set_prop (SCM s_proxy,
   gchar *prop_name;
   GValue *value;
 
-  SCM_ASSERT (SCM_IMP (s_proxy),  s_proxy,  SCM_ARG1, "bse-set-prop");
-  SCM_ASSERT (SCM_STRINGP (s_prop_name), s_prop_name, SCM_ARG2, "bse-set-prop");
+  SCM_ASSERT (SCM_IS_GLUE_PROXY (s_proxy),  s_proxy,  SCM_ARG1, "bse-glue-set-prop");
+  SCM_ASSERT (SCM_STRINGP (s_prop_name), s_prop_name, SCM_ARG2, "bse-glue-set-prop");
 
   BSE_SCM_DEFER_INTS ();
 
-  proxy = gh_scm2long (s_proxy);
+  proxy = SCM_GET_GLUE_PROXY (s_proxy);
   prop_name = g_strndup (SCM_ROCHARS (s_prop_name), SCM_LENGTH (s_prop_name));
   bse_scm_enter_gc (&gclist, prop_name, g_free, SCM_LENGTH (s_prop_name));
 
@@ -453,12 +509,12 @@ bse_scm_glue_get_prop (SCM s_proxy,
   gchar *prop_name;
   const GValue *value;
 
-  SCM_ASSERT (SCM_IMP (s_proxy),  s_proxy,  SCM_ARG1, "bse-get-prop");
-  SCM_ASSERT (SCM_STRINGP (s_prop_name), s_prop_name, SCM_ARG2, "bse-get-prop");
+  SCM_ASSERT (SCM_IS_GLUE_PROXY (s_proxy),  s_proxy,  SCM_ARG1, "bse-glue-get-prop");
+  SCM_ASSERT (SCM_STRINGP (s_prop_name), s_prop_name, SCM_ARG2, "bse-glue-get-prop");
 
   BSE_SCM_DEFER_INTS ();
 
-  proxy = gh_scm2long (s_proxy);
+  proxy = SCM_GET_GLUE_PROXY (s_proxy);
   prop_name = g_strndup (SCM_ROCHARS (s_prop_name), SCM_LENGTH (s_prop_name));
   bse_scm_enter_gc (&gclist, prop_name, g_free, SCM_LENGTH (s_prop_name));
 
@@ -523,41 +579,37 @@ bse_scm_glue_call (SCM s_proc_name,
 }
 
 typedef struct {
-  gulong proxy;
-  gchar *signal;
-  SCM s_lambda;
-  const SfiSeq *tmp_args;
-} SigData;
+  gulong        proxy;
+  gchar        *signal;
+  SCM           s_lambda;
+  guint         n_args;
+  const GValue *args;
+} SignalData;
 
 static void
-signal_handler_destroyed (gpointer data)
+signal_data_free (gpointer  data,
+                  GClosure *closure)
 {
-  SigData *sdata = data;
-
-  scm_unprotect_object (sdata->s_lambda);
-  sdata->s_lambda = SCM_EOL;
+  SignalData *sdata = data;
+  SCM s_lambda = sdata->s_lambda;
   g_free (sdata->signal);
   g_free (sdata);
+  scm_gc_unprotect_object (s_lambda);
 }
 
 static SCM
-marshal_sproc (void *data)
+signal_marshal_sproc (void *data)
 {
-  SigData *sdata = data;
+  SignalData *sdata = data;
   SCM s_ret, args = SCM_EOL;
-  const SfiSeq *seq = sdata->tmp_args;
   guint i;
 
-  sdata->tmp_args = NULL;
+  i = sdata->n_args;
+  g_return_val_if_fail (sdata->n_args > 0, SCM_UNSPECIFIED);
+  sdata->n_args = 0;
 
-  g_return_val_if_fail (seq != NULL && seq->n_elements > 0, SCM_UNSPECIFIED);
-
-  i = seq->n_elements;
   while (i--)
-    {
-      SCM arg = bse_scm_from_value (seq->elements + i);
-      args = gh_cons (arg, args);
-    }
+    args = gh_cons (bse_scm_from_value (sdata->args + i), args);
 
   s_ret = scm_apply (sdata->s_lambda, args, SCM_EOL);
 
@@ -565,16 +617,19 @@ marshal_sproc (void *data)
 }
 
 static void
-signal_handler (gpointer      sig_data,
-		const gchar  *signal,
-		const SfiSeq *args)
+signal_closure_marshal (GClosure       *closure,
+                        GValue         *return_value,
+                        guint           n_param_values,
+                        const GValue   *param_values,
+                        gpointer        invocation_hint,
+                        gpointer        marshal_data)
 {
   SCM_STACKITEM stack_item;
-  SigData *sdata = sig_data;
-
-  sdata->tmp_args = args;
-  scm_internal_cwdr ((scm_catch_body_t) marshal_sproc, sdata,
-		     scm_handle_by_message_noexit, "BSE", &stack_item);
+  SignalData *sdata = closure->data;
+  sdata->n_args = n_param_values;
+  sdata->args = param_values;
+  scm_internal_cwdr ((scm_catch_body_t) signal_marshal_sproc, sdata,
+                     scm_handle_by_message_noexit, "BSE", &stack_item);
 }
 
 SCM
@@ -582,27 +637,52 @@ bse_scm_signal_connect (SCM s_proxy,
 			SCM s_signal,
 			SCM s_lambda)
 {
-  gulong proxy, id;
-  SigData *sdata;
-  
-  SCM_ASSERT (SCM_IMP (s_proxy), s_proxy,  SCM_ARG1, "bse-signal-connect");
+  SfiProxy proxy;
+  gulong id;
+  SignalData *sdata;
+  GClosure *closure;
+
+  SCM_ASSERT (SCM_IS_GLUE_PROXY (s_proxy), s_proxy,  SCM_ARG1, "bse-signal-connect");
+  proxy = SCM_GET_GLUE_PROXY (s_proxy);
+
   SCM_ASSERT (SCM_STRINGP (s_signal), s_signal, SCM_ARG2, "bse-signal-connect");
   SCM_ASSERT (gh_procedure_p (s_lambda), s_lambda,  SCM_ARG3, "bse-signal-connect");
 
-  proxy = gh_scm2ulong (s_proxy);
+  scm_gc_protect_object (s_lambda);
 
   BSE_SCM_DEFER_INTS ();
-  sdata = g_new0 (SigData, 1);
+  sdata = g_new0 (SignalData, 1);
   sdata->proxy = proxy;
   sdata->signal = g_strndup (SCM_ROCHARS (s_signal), SCM_LENGTH (s_signal));
   sdata->s_lambda = s_lambda;
-  scm_protect_object (sdata->s_lambda);
-  id = 0; // FIXME: id = sfi_glue_signal_connect (proxy, sdata->signal, signal_handler, sdata, signal_handler_destroyed);
+  closure = g_closure_new_simple (sizeof (GClosure), sdata);
+  g_closure_add_finalize_notifier (closure, sdata, signal_data_free);
+  g_closure_set_marshal (closure, signal_closure_marshal);
+  id = sfi_glue_signal_connect_closure (proxy, sdata->signal, closure, NULL);
   BSE_SCM_ALLOW_INTS ();
   
   return gh_ulong2scm (id);
 }
 
+SCM
+bse_scm_signal_disconnect (SCM s_proxy,
+                           SCM s_handler_id)
+{
+  SfiProxy proxy;
+  gulong id;
+
+  SCM_ASSERT (SCM_IS_GLUE_PROXY (s_proxy), s_proxy,  SCM_ARG1, "bse-signal-disconnect");
+  proxy = SCM_GET_GLUE_PROXY (s_proxy);
+
+  id = scm_num2ulong (s_handler_id, SCM_ARG2, "bse-signal-disconnect");
+  if (id < 1)
+    scm_out_of_range ("bse-signal-disconnect", s_handler_id);
+
+  sfi_glue_signal_disconnect (proxy, id);
+
+  return SCM_UNSPECIFIED;
+}
+
 static gboolean script_register_enabled = FALSE;
 
 void
@@ -726,7 +806,11 @@ bse_scm_context_iteration (SCM s_may_block)
     {
       /* FIXME: we need a real poll() based wait implementation here */
       do
-	g_usleep (250 * 1000);
+        {
+          BSE_SCM_ALLOW_INTS ();
+          g_usleep (250 * 1000);
+          BSE_SCM_DEFER_INTS ();
+        }
       while (!sfi_glue_context_pending ());
       sfi_glue_context_dispatch ();
     }
@@ -778,21 +862,23 @@ bse_scm_interp_init (void)
   gchar **procs, *procs2[2];
   guint i;
 
-  tc_gc_cell = scm_make_smob_type ("BseScmGCCell", 0);
-  scm_set_smob_mark (tc_gc_cell, bse_scm_mark_gc_cell);
-  scm_set_smob_free (tc_gc_cell, bse_scm_free_gc_cell);
+  tc_glue_gc_cell = scm_make_smob_type ("BseScmGCCell", 0);
+  scm_set_smob_mark (tc_glue_gc_cell, bse_scm_mark_gc_cell);
+  scm_set_smob_free (tc_glue_gc_cell, bse_scm_free_gc_cell);
 
   tc_glue_gc_plateau = scm_make_smob_type ("BseScmGcPlateau", 0);
   scm_set_smob_free (tc_glue_gc_plateau, bse_scm_gc_plateau_free);
 
   tc_glue_rec = scm_make_smob_type ("BseGlueRec", 0);
   scm_set_smob_free (tc_glue_rec, bse_scm_free_glue_rec);
-  gh_new_procedure ("bse-record-get", bse_scm_glue_rec_get, 2, 0, 0);
-  gh_new_procedure ("bse-record-print", bse_scm_glue_rec_print, 1, 0, 0);
+  gh_new_procedure ("bse-rec-get", bse_scm_glue_rec_get, 2, 0, 0);
+  gh_new_procedure ("bse-rec-set", bse_scm_glue_rec_set, 3, 0, 0);
+  gh_new_procedure ("bse-rec-new", bse_scm_glue_rec_new, 0, 1, 0);
+  gh_new_procedure ("bse-rec-print", bse_scm_glue_rec_print, 1, 0, 0);
 
   tc_glue_proxy = scm_make_smob_type ("SfiProxy", 0);
   SCM_NEWSMOB (glue_null_proxy, tc_glue_proxy, 0);
-  scm_protect_object (glue_null_proxy);
+  scm_permanent_object (glue_null_proxy);
   scm_set_smob_equalp (tc_glue_proxy, bse_scm_proxy_equalp);
   gh_new_procedure ("bse-null-proxy?", bse_scm_null_proxyp, 1, 0, 0);
 
@@ -820,6 +906,7 @@ bse_scm_interp_init (void)
   gh_new_procedure ("bse-script-fetch-args", bse_scm_script_args, 0, 0, 0);
   // FIXME: gh_new_procedure ("bse-enum-match?", bse_scm_enum_match, 2, 0, 0);
   gh_new_procedure ("bse-signal-connect", bse_scm_signal_connect, 3, 0, 0);
+  gh_new_procedure ("bse-signal-disconnect", bse_scm_signal_disconnect, 2, 0, 0);
   gh_new_procedure ("bse-context-pending", bse_scm_context_pending, 0, 0, 0);
   gh_new_procedure ("bse-context-iteration", bse_scm_context_iteration, 1, 0, 0);
 }
